@@ -36,7 +36,8 @@ from types           import NoneType
 
 from notify.mediator import AbstractMediator
 from notify.signal   import AbstractSignal, Signal
-from notify.utils    import is_valid_identifier, raise_not_implemented_exception
+from notify.utils    import is_callable, is_valid_identifier, mangle_identifier, as_string, \
+                            raise_not_implemented_exception
  
 try:
     import contextlib
@@ -84,8 +85,7 @@ class AbstractValueObject (object):
     _additional_description
     """
 
-    __slots__ = ('__weakref__',
-                 '_AbstractValueObject__signal', '_AbstractValueObject__freeze_flag')
+    __slots__ = ('__weakref__', as_string.__signal, as_string.__freeze_flag)
 
 
     # Implementation note: `__freeze_flag' can be None (not frozen) or True (frozen.)
@@ -319,16 +319,16 @@ class AbstractValueObject (object):
         """
 
         if not isinstance (value_object, AbstractValueObject):
-            raise TypeError ("can only synchronize with other `AbstractValueObject' instances")
+            raise TypeError ("can only synchronize with other 'AbstractValueObject' instances")
 
         if value_object is self:
-            raise ValueError ("can only synchronize with other `AbstractValueObject' instances")
+            raise ValueError ("can only synchronize with other 'AbstractValueObject' instances")
 
         if not self._is_mutable ():
-            raise ValueError ("`%s' is not mutable" % self)
+            raise ValueError ("'%s' is not mutable" % self)
 
         if not value_object._is_mutable ():
-            raise ValueError ("`%s' is not mutable" % value_object)
+            raise ValueError ("'%s' is not mutable" % value_object)
 
         if mediator is None:
             # Note: order is important!
@@ -366,13 +366,13 @@ class AbstractValueObject (object):
         """
 
         if not isinstance (value_object, AbstractValueObject):
-            raise TypeError ("can only synchronize with other `AbstractValueObject' instances")
+            raise TypeError ("can only synchronize with other 'AbstractValueObject' instances")
 
         if value_object is self:
-            raise ValueError ("can only synchronize with other `AbstractValueObject' instances")
+            raise ValueError ("can only synchronize with other 'AbstractValueObject' instances")
 
         if not value_object._is_mutable ():
-            raise ValueError ("target `AbstractValueObject' instance is not mutable")
+            raise ValueError ("target 'AbstractValueObject' instance is not mutable")
 
         if mediator is None:
             # Note: order is important!
@@ -737,6 +737,14 @@ class AbstractValueObject (object):
                                inside the created instance.  It will be used for calling
                                C{getter} and C{setter} functions.
 
+        @option:               C{property} — valid Python identifier.  If specified,
+                               C{object}’s value will be readable through a property, but
+                               not writable.
+
+        @option:               C{dict} — if true, derived type will have a C{__dict__}
+                               slot, allowing setting any attribute on it.  This is
+                               silently ignored if this class objects already have a dict.
+
         @option:               C{getter} — a callable accepting one argument, whose return
                                value will be used as C{L{get}} method result.  If
                                C{object} option is specified, the only argument will be
@@ -757,22 +765,22 @@ class AbstractValueObject (object):
         """
 
         if not is_valid_identifier (new_class_name):
-            raise TypeError ("`%s' is not a valid Python identifier" % new_class_name)
+            raise TypeError ("'%s' is not a valid Python identifier" % new_class_name)
 
         full_options                   = dict (options)
         full_options['cls']            = cls
         full_options['new_class_name'] = new_class_name
 
-        dictionary = {}
+        dictionary = { '__slots__': () }
 
         for value in cls._generate_derived_type_dictionary (full_options):
             if value[0] != '__slots__':
                 dictionary[value[0]] = value[1]
             else:
-                if not '__slots__' in dictionary:
-                    dictionary['__slots__'] = ()
-
-                dictionary['__slots__'] += tuple (value[1])
+                if isinstance (value[1], basestring):
+                    dictionary['__slots__'] += (value[1],)
+                else:
+                    dictionary['__slots__'] += tuple (value[1])
 
         try:
             return type (new_class_name, (cls,), dictionary)
@@ -812,7 +820,8 @@ class AbstractValueObject (object):
             ...     functions = {}
             ...
             ...     if 'foo_value' in options:
-            ...         exec 'def foo (self): return foo_value' in options, functions
+            ...         exec 'def foo (self): return foo_value' \
+            ...              in { 'foo_value': options['foo_value'] }, functions
             ...
             ...     ...
             ...
@@ -821,7 +830,13 @@ class AbstractValueObject (object):
 
         Returned value for C{__slots__} is treated specially.  While normally values
         associated with the same name override previous values, values for C{__slots__}
-        are combined in to a tuple instead.
+        are combined into a tuple instead.
+
+        Note that it is not recommended to use C{options} for execution globals or locals
+        dictionary directly.  This way your code may become vulnerable to other option
+        addition, e.g. for some derivative of the class.  For instance, you may use
+        C{property} built-in, then setting it in C{options} will hide the built-in from
+        your code.  Consider using C{L{_filter_options}} utility method.
 
         @param options:    dictionary of options passed to C{L{derive_type}} method, plus
                            C{cls} and C{new_class_name}.
@@ -838,37 +853,59 @@ class AbstractValueObject (object):
         if 'object' in options:
             object = options['object']
             if not is_valid_identifier (object):
-                raise ValueError ("`%s' is not a valid Python identifier" % object)
+                raise ValueError ("'%s' is not a valid Python identifier" % object)
 
-            yield '__slots__', ('_%s__%s' % (options['new_class_name'].lstrip ('_'), object),)
+            yield '__slots__', mangle_identifier (options['new_class_name'], object)
 
             exec (('def __init__(self, %s):\n'
                    '    cls.__init__(self)\n'
                    '    %s = %s')
                   % (object, AbstractValueObject._get_object (options), object)) \
-                  in options, functions
+                  in functions
 
-            del object
+            if 'property' in options:
+                property = options['property']
+                if property == object:
+                    raise ValueError ("'property' option cannot be the same as 'object'")
+
+                if not is_valid_identifier (property):
+                    raise ValueError ("'%s' is not a valid Python identifier" % property)
+
+                exec ('%s = property (lambda self: %s)'
+                      % (mangle_identifier (options['new_class_name'], property),
+                         AbstractValueObject._get_object (options))) \
+                      in functions
+
+        else:
+            if 'property' in options:
+                raise ValueError ("'property' without 'object' option doesn't make sense")
+
+        if 'dict' in options and options['dict']:
+            # Gracefully ignore if this type already has a dict.
+            if not cls.__dictoffset__:
+                yield '__slots__', '__dict__'
+
+        filtered_options = AbstractValueObject._filter_options (options, 'getter', 'setter')
 
         if 'getter' in options:
-            if not callable (options['getter']):
-                raise TypeError ("`getter' must be a callable")
+            if not is_callable (options['getter']):
+                raise TypeError ("'getter' must be a callable")
 
             exec ('def get (self): return getter (%s)'
-                  % AbstractValueObject._get_object (options)) in options, functions
+                  % AbstractValueObject._get_object (options)) \
+                  in filtered_options, functions
 
         if 'setter' in options:
-            if not callable (options['setter']):
-                raise TypeError ("`setter' must be a callable")
+            if not is_callable (options['setter']):
+                raise TypeError ("'setter' must be a callable")
 
             exec ('def set (self, value): return setter (%s, value)'
-                  % AbstractValueObject._get_object (options)) in options, functions
+                  % AbstractValueObject._get_object (options)) \
+                  in filtered_options, functions
 
         for function in functions.iteritems ():
             yield function
 
-        del functions
-
 
     def _get_object (options):
         """
@@ -887,14 +924,39 @@ class AbstractValueObject (object):
         """
 
         if 'object' in options:
-            return 'self._%s__%s' % (options['new_class_name'].lstrip ('_'), options['object'])
+            return 'self.%s' % mangle_identifier (options['new_class_name'], options['object'])
         else:
             return 'self'
 
 
+    def _filter_options (options, *names):
+        """
+        Return a subset of C{options} including only those listed in C{names}.  This is a
+        utility method.  Its main purpose is to build a subset of options dictionary for
+        C{L{_generate_derived_type_dictionary}} method, which is safe to use as locals or
+        globals for evaluating method definitions.
+
+        @param options: full dictionary of options.
+        @type  options: C{dict}
+
+        @param names:   permitted options.
+
+        @rtype:         C{dict}
+        """
+
+        filtered_options = {}
+
+        for name in names:
+            if name in options:
+                filtered_options[name] = options[name]
+
+        return filtered_options
+
+
     derive_type                       = classmethod  (derive_type)
     _generate_derived_type_dictionary = classmethod  (_generate_derived_type_dictionary)
     _get_object                       = staticmethod (_get_object)
+    _filter_options                   = staticmethod (_filter_options)
 
 
 
