@@ -39,7 +39,7 @@ import weakref
 from notify.base   import AbstractValueObject
 from notify.gc     import AbstractGCProtector
 from notify.signal import CleanSignal
-from notify.utils  import raise_not_implemented_exception, DummyReference
+from notify.utils  import is_callable, as_string, raise_not_implemented_exception, DummyReference
 
 
 
@@ -256,7 +256,7 @@ class AbstractCondition (AbstractValueObject):
             else:
                 return true_condition
         else:
-            raise TypeError ("`true_condition' and `false_condition' must be conditions")
+            raise TypeError ("'true_condition' and 'false_condition' must be conditions")
 
 
 
@@ -271,7 +271,7 @@ class AbstractStateTrackingCondition (AbstractCondition):
     internal ones inherit C{L{AbstractCondition}} directly instead.
     """
 
-    __slots__ = ('_AbstractStateTrackingCondition__state')
+    __slots__ = (as_string.__state)
 
 
     def __init__(self, initial_state):
@@ -317,13 +317,18 @@ class AbstractStateTrackingCondition (AbstractCondition):
         @returns:      Whether condition state changed as a result.
         """
 
-        state = bool (value)
-
-        if self.get () != state:
-            self.__state = state
-            return self._value_changed (state)
+        # This seems to be a little faster than more generic _set() as in variable branch.
+        # Only possible since there are just two possible states.
+        if value:
+            if not self.get ():
+                self.__state = True
+                return self._value_changed (True)
         else:
-            return False
+            if self.get ():
+                self.__state = False
+                return self._value_changed (False)
+
+        return False
 
 
     def _generate_derived_type_dictionary (cls, options):
@@ -332,8 +337,9 @@ class AbstractStateTrackingCondition (AbstractCondition):
             if attribute[0] not in ('get', 'set'):
                 yield attribute
 
-        functions = {}
-        object    = options.get ('object')
+        functions        = {}
+        object           = options.get ('object')
+        filtered_options = AbstractValueObject._filter_options (options, 'cls', 'getter', 'setter')
 
         if 'getter' in options:
             if object is not None:
@@ -341,14 +347,14 @@ class AbstractStateTrackingCondition (AbstractCondition):
                        '    cls.__init__(self, getter (%s))\n'
                        '    %s = %s')
                       % (object, object, AbstractValueObject._get_object (options), object)) \
-                      in options, functions
+                      in filtered_options, functions
             else:
                 exec ('def __init__(self):\n'
-                      '    cls.__init__(self, getter (self))\n') in options, functions
+                      '    cls.__init__(self, getter (self))\n') in filtered_options, functions
 
             exec (('def resynchronize_with_backend (self):\n'
                    '    self._set (getter (%s))')
-                  % AbstractValueObject._get_object (options)) in options, functions
+                  % AbstractValueObject._get_object (options)) in filtered_options, functions
 
         else:
             if 'setter' in options:
@@ -364,13 +370,13 @@ class AbstractStateTrackingCondition (AbstractCondition):
                        '    %s\n')
                       % (object, AbstractValueObject._get_object (options), object,
                          setter_statement)) \
-                      in options, functions
+                      in filtered_options, functions
             else:
                 exec (('def __init__(self, initial_state):\n'
                        '    cls.__init__(self, initial_state)\n'
                        '    %s\n')
                       % setter_statement) \
-                      in options, functions
+                      in filtered_options, functions
 
         if 'setter' in options:
             exec (('def _set (self, value):\n'
@@ -381,16 +387,13 @@ class AbstractStateTrackingCondition (AbstractCondition):
                    '        return self._value_changed (state)\n'
                    '    else:\n'
                    '        return False')
-                  % AbstractValueObject._get_object (options)) in options, functions
+                  % AbstractValueObject._get_object (options)) in filtered_options, functions
 
-            exec 'def set (self, value): return self._set (value)' in options, functions
+            exec 'def set (self, value): return self._set (value)' in functions
 
         for function in functions.iteritems ():
             yield function
 
-        del functions
-        del object
-
 
     _generate_derived_type_dictionary = classmethod  (_generate_derived_type_dictionary)
 
@@ -445,7 +448,7 @@ class PredicateCondition (AbstractStateTrackingCondition):
     conditions remember the way to evaluate their state.
     """
 
-    __slots__ = ('_PredicateCondition__predicate')
+    __slots__ = (as_string.__predicate)
 
 
     def __init__(self, predicate, initial_object):
@@ -465,7 +468,7 @@ class PredicateCondition (AbstractStateTrackingCondition):
         @raises exception:     whatever C{predicate} raises, if anything.
         """
 
-        if not callable (predicate):
+        if not is_callable (predicate):
             raise TypeError ('predicate must be callable')
 
         super (PredicateCondition, self).__init__(predicate (initial_object))
@@ -495,7 +498,7 @@ class PredicateCondition (AbstractStateTrackingCondition):
 
 
     def _generate_derived_type_dictionary (cls, options):
-        raise TypeError ("`PredicateCondition' doesn't support derive_type() method")
+        raise TypeError ("'PredicateCondition' doesn't support derive_type() method")
 
     _generate_derived_type_dictionary = classmethod (_generate_derived_type_dictionary)
 
@@ -522,7 +525,7 @@ class WatcherCondition (AbstractStateTrackingCondition):
     """
 
 
-    __slots__ = ('_WatcherCondition__watched_condition')
+    __slots__ = (as_string.__watched_condition)
 
 
     def __init__(self, condition_to_watch = None):
@@ -647,7 +650,7 @@ class WatcherCondition (AbstractStateTrackingCondition):
 
 
     def _generate_derived_type_dictionary (cls, options):
-        raise TypeError ("`PredicateCondition' doesn't support derive_type() method")
+        raise TypeError ("'WatcherCondition' doesn't support derive_type() method")
 
     _generate_derived_type_dictionary = classmethod (_generate_derived_type_dictionary)
 
@@ -694,7 +697,7 @@ class _True (AbstractCondition):
              and isinstance (false_condition, AbstractCondition)):
             return true_condition
         else:
-            raise TypeError ("`true_condition' and `false_condition' must be conditions")
+            raise TypeError ("'true_condition' and 'false_condition' must be conditions")
 
 
     def __repr__(self):
@@ -746,7 +749,7 @@ class _False (AbstractCondition):
              and isinstance (false_condition, AbstractCondition)):
             return false_condition
         else:
-            raise TypeError ("`true_condition' and `false_condition' must be conditions")
+            raise TypeError ("'true_condition' and 'false_condition' must be conditions")
 
 
     def __repr__(self):
@@ -764,20 +767,30 @@ AbstractCondition.FALSE = _False ()
 
 
 
-class _Not (AbstractStateTrackingCondition):
+# Previously derived from `AbstractStateTrackingCondition', but this is a tiny little bit
+# more efficient.
 
-    __slots__ = ('_Not__negated_condition')
+class _Not (AbstractCondition):
+
+    # We need to save our state, since `negated_condition' may be gc-collected.
+    __slots__ = (as_string.__state, as_string.__negated_condition)
 
 
     def __init__(self, negated_condition):
-        super (_Not, self).__init__(not negated_condition)
+        super (_Not, self).__init__()
 
+        self.__state             = not negated_condition
         self.__negated_condition = weakref.ref (negated_condition, self.__on_usage_change)
+
         negated_condition.changed.connect (self.__on_negated_condition_change)
 
 
+    def get (self):
+        return self.__state
+
     def __on_negated_condition_change (self, new_state):
-        self._set (not new_state)
+        self.__state = not new_state
+        self._value_changed (not new_state)
 
 
     def __get_negated_condition (self):
@@ -786,7 +799,7 @@ class _Not (AbstractStateTrackingCondition):
         if negated_condition is not None:
             return negated_condition
         else:
-            return AbstractCondition.to_constant (self.get ())
+            return AbstractCondition.to_constant (self.__state)
 
 
     def _create_signal (self):
@@ -826,14 +839,15 @@ class _Not (AbstractStateTrackingCondition):
 
 class _Binary (AbstractCondition):
 
-    __slots__ = ('_Binary__condition1', '_Binary__condition2', '_term_state')
+    __slots__ = (as_string.__condition1, as_string.__condition2, '_term_state')
 
 
     def __init__(self, condition1, condition2):
         super (_Binary, self).__init__()
 
-        self.__condition1 = weakref.ref (condition1, self.__on_usage_change)
-        self.__condition2 = weakref.ref (condition2, self.__on_usage_change)
+        on_usage_change   = self.__on_usage_change
+        self.__condition1 = weakref.ref (condition1, on_usage_change)
+        self.__condition2 = weakref.ref (condition2, on_usage_change)
 
         condition1.changed.connect (self._on_term1_change)
         condition2.changed.connect (self._on_term2_change)
@@ -970,7 +984,7 @@ class _Xor (_Binary):
 
 class _IfElse (AbstractCondition):
 
-    __slots__ = ('_IfElse__if', '_IfElse__then', '_IfElse__else', '_IfElse__term_state')
+    __slots__ = (as_string.__if, as_string.__then, as_string.__else, as_string.__term_state)
 
 
     __TERM_STATE_TO_SELF_STATE = (False, True, False, True, False, False, True, True)
@@ -979,9 +993,10 @@ class _IfElse (AbstractCondition):
     def __init__(self, _if, _then, _else):
         super (_IfElse, self).__init__()
 
-        self.__if         = weakref.ref (_if,   self.__on_usage_change)
-        self.__then       = weakref.ref (_then, self.__on_usage_change)
-        self.__else       = weakref.ref (_else, self.__on_usage_change)
+        on_usage_change   = self.__on_usage_change
+        self.__if         = weakref.ref (_if,   on_usage_change)
+        self.__then       = weakref.ref (_then, on_usage_change)
+        self.__else       = weakref.ref (_else, on_usage_change)
         self.__term_state = (_if.get () * 4 + _then.get () * 2 + _else.get ())
 
         _if  .changed.connect (self.__on_if_term_change)
