@@ -42,7 +42,10 @@ struct
   PyObject *  protected_objects_dict;
   long int    num_active_protections;
 }
-DebugGCProtector;
+RaisingGCProtector;
+
+
+typedef  RaisingGCProtector  DebugGCProtector;
 
 
 
@@ -66,20 +69,23 @@ static PyObject *   FastGCProtector_unprotect       (FastGCProtector *self,
 static PyObject *   FastGCProtector_get_num_active_protections
                       (FastGCProtector *self);
 
-static int          DebugGCProtector_init           (DebugGCProtector *self,
+static int          RaisingGCProtector_init         (RaisingGCProtector *self,
                                                      PyObject *arguments, PyObject *keywords);
-static void         DebugGCProtector_dealloc        (DebugGCProtector *self);
-static PyObject *   DebugGCProtector_protect        (DebugGCProtector *self,
+static void         RaisingGCProtector_dealloc      (RaisingGCProtector *self);
+static PyObject *   RaisingGCProtector_protect      (RaisingGCProtector *self,
                                                      PyObject *arguments, PyObject *keywords);
+static PyObject *   RaisingGCProtector_unprotect    (RaisingGCProtector *self,
+                                                     PyObject *arguments, PyObject *keywords);
+static PyObject *   RaisingGCProtector_get_num_object_protections
+                                                    (RaisingGCProtector *self,
+                                                     PyObject *arguments, PyObject *keywords);
+static PyObject *   RaisingGCProtector_get_num_protected_objects
+                      (RaisingGCProtector *self);
+static PyObject *   RaisingGCProtector_get_num_active_protections
+                      (RaisingGCProtector *self);
+
 static PyObject *   DebugGCProtector_unprotect      (DebugGCProtector *self,
                                                      PyObject *arguments, PyObject *keywords);
-static PyObject *   DebugGCProtector_get_num_object_protections
-                                                    (DebugGCProtector *self,
-                                                     PyObject *arguments, PyObject *keywords);
-static PyObject *   DebugGCProtector_get_num_protected_objects
-                      (DebugGCProtector *self);
-static PyObject *   DebugGCProtector_get_num_active_protections
-                      (DebugGCProtector *self);
 
 
 
@@ -94,7 +100,7 @@ don't need a reference to notice a state change.\n\
 \n\
 This module defines both a simple L{interface <AbstractGCProtector>} and several \
 implementations, some, which are suitable for production use (C{L{FastGCProtector}}), \
-some for debugging purposes (C{L{DebugGCProtector}}.)\n\
+some for debugging purposes (C{L{RaisingGCProtector}}, C{L{DebugGCProtector}}.)\n\
 \n\
 Py-notify classes use value of the C{L{AbstractGCProtector.default}} variable as the \
 protector instance.  In case you run into a problem, \
@@ -104,6 +110,13 @@ However, we believe that Py-notify classes must not cause problems themselves, t
 only if you use a garbage-collection protector yourself."
 
 
+#define UNPROTECTION_ERROR_DOC "\
+Error that is raised by some L{AbstractGCProtector <garbage-collection protectors>} when you try \
+to L{AbstractGCProtector.unprotect <unprotect>} an object more times than it had been \
+L{AbstractGCProtector.protect <protected>}.  Of the standard protectors only \
+C{L{RaisingGCProtector}} ever raises these exceptions."
+
+
 #define ABSTRACT_GC_PROTECTOR_DOC                                       \
   NULL
 
@@ -146,7 +159,8 @@ problems, e.g. with a C{L{DebugGCProtector}}."
 #define FAST_GC_PROTECTOR_DOC "\
 Default fast implementation of C{AbstractGCProtector} interface.  It is suitable for \
 production use, but difficult to debug problems with, because it doesn't track what has and \
-what has not be protected.  For that purpose, use C{L{DebugGCProtector}}."
+what has not be protected.  For that purpose, use C{L{RaisingGCProtector}} or \
+C{L{DebugGCProtector}}."
 
 #define FAST_GC_PROTECTOR_PROTECT_DOC                                   \
   NULL
@@ -158,40 +172,57 @@ what has not be protected.  For that purpose, use C{L{DebugGCProtector}}."
   NULL
 
 
-#define DEBUG_GC_PROTECTOR_DOC "\
-Implementation of C{AbstractGCProtector} interface suitable for debugging possible problems. \
-Instances of this class track what they have protected so far and how many times.  If you \
-try to unprotect an object more times than it has been protected, a stack trace will be \
-printed and nothing will be done.  Note that no exception will be thrown.\n\
+#define RAISING_GC_PROTECTOR_DOC "\
+Implementation of C{AbstractGCProtector} interface suitable for aggressively debugging \
+possible problems.  Instances of this class track what they have protected so far and how \
+many times.  If you try to unprotect an object more times than it has been protected, an \
+exception will be raised.\n\
 \n\
 There is also a number of functions and properties in this class that allow you to retrieve \
-various protection information."
+various protection information.\n\
+\n\
+@see: C{L{DebugGCProtector}}"
 
-#define DEBUG_GC_PROTECTOR_PROTECT_DOC                                  \
+#define RAISING_GC_PROTECTOR_PROTECT_DOC                                \
   NULL
 
+#define RAISING_GC_PROTECTOR_UNPROTECT_DOC                              \
+  NULL
+
+#define RAISING_GC_PROTECTOR_GET_NUM_OBJECT_PROTECTIONS                 \
+  NULL
+
+#define RAISING_GC_PROTECTOR_NUM_PROTECTED_OBJECTS_DOC                  \
+  NULL
+
+#define RAISING_GC_PROTECTOR_NUM_ACTIVE_PROTECTIONS_DOC                 \
+  NULL
+
+
+#define DEBUG_GC_PROTECTOR_DOC "\
+Implementation of C{AbstractGCProtector} interface suitable for debugging possible problems. \
+Instances of this class track what they have protected so far and how many times.  If you try to \
+unprotect an object more times than it has been protected, a stack trace will be printed and \
+nothing will be done.  Note that unlike C{L{RaisingGCProtector}}, no exception will be thrown.\n\
+\n\
+There is also a number of functions and properties in this class that allow you to retrieve \
+various protection information.\n\
+\n\
+@see: C{L{RaisingGCProtector}}"
+
 #define DEBUG_GC_PROTECTOR_UNPROTECT_DOC                                \
   NULL
 
-#define DEBUG_GC_PROTECTOR_GET_NUM_OBJECT_PROTECTIONS                   \
-  NULL
-
-#define DEBUG_GC_PROTECTOR_NUM_PROTECTED_OBJECTS_DOC                    \
-  NULL
-
-#define DEBUG_GC_PROTECTOR_NUM_ACTIVE_PROTECTIONS_DOC                   \
-  NULL
-
 
 
 /*- Types ----------------------------------------------------------*/
 
 PyMethodDef  AbstractGCProtector_methods[]
-  = { { "protect",     (PyCFunction)  AbstractGCProtector_protect,
+  = { { "protect",     (PyCFunction) AbstractGCProtector_protect,
         METH_VARARGS | METH_KEYWORDS,             ABSTRACT_GC_PROTECTOR_PROTECT_DOC },
-      { "unprotect",   (PyCFunction)  AbstractGCProtector_unprotect,
+      { "unprotect",   (PyCFunction) AbstractGCProtector_unprotect,
         METH_VARARGS | METH_KEYWORDS,               ABSTRACT_GC_PROTECTOR_UNPROTECT_DOC },
-      { "set_default", (PyCFunction)  AbstractGCProtector_set_default,
+      { "set_default", (PyCFunction) AbstractGCProtector_set_default,
         METH_VARARGS | METH_KEYWORDS | METH_STATIC, ABSTRACT_GC_PROTECTOR_SET_DEFAULT_DOC },
       { NULL, NULL, 0, NULL } };
 
@@ -242,9 +273,9 @@ PyTypeObject  AbstractGCProtector_Type
 
 
 PyMethodDef  FastGCProtector_methods[]
-  = { { "protect",     (PyCFunction)  FastGCProtector_protect,
+  = { { "protect",     (PyCFunction) FastGCProtector_protect,
         METH_VARARGS | METH_KEYWORDS, FAST_GC_PROTECTOR_PROTECT_DOC },
-      { "unprotect",   (PyCFunction)  FastGCProtector_unprotect,
+      { "unprotect",   (PyCFunction) FastGCProtector_unprotect,
         METH_VARARGS | METH_KEYWORDS, FAST_GC_PROTECTOR_UNPROTECT_DOC },
       { NULL, NULL, 0, NULL } };
 
@@ -299,29 +330,80 @@ PyTypeObject  FastGCProtector_Type
     };
 
 
-PyMethodDef  DebugGCProtector_methods[]
-  = { { "protect",     (PyCFunction)  DebugGCProtector_protect,
-        METH_VARARGS | METH_KEYWORDS, DEBUG_GC_PROTECTOR_PROTECT_DOC },
-      { "unprotect",   (PyCFunction)  DebugGCProtector_unprotect,
-        METH_VARARGS | METH_KEYWORDS, DEBUG_GC_PROTECTOR_UNPROTECT_DOC },
-      { "get_num_object_protections", (PyCFunction) DebugGCProtector_get_num_object_protections,
-        METH_VARARGS | METH_KEYWORDS, DEBUG_GC_PROTECTOR_GET_NUM_OBJECT_PROTECTIONS },
+PyMethodDef  RaisingGCProtector_methods[]
+  = { { "protect",     (PyCFunction) RaisingGCProtector_protect,
+        METH_VARARGS | METH_KEYWORDS, RAISING_GC_PROTECTOR_PROTECT_DOC },
+      { "unprotect",   (PyCFunction) RaisingGCProtector_unprotect,
+        METH_VARARGS | METH_KEYWORDS, RAISING_GC_PROTECTOR_UNPROTECT_DOC },
+      { "get_num_object_protections", (PyCFunction) RaisingGCProtector_get_num_object_protections,
+        METH_VARARGS | METH_KEYWORDS, RAISING_GC_PROTECTOR_GET_NUM_OBJECT_PROTECTIONS },
       { NULL, NULL, 0, NULL } };
 
-PyGetSetDef  DebugGCProtector_properties[]
-  = { { "num_protected_objects", (getter) DebugGCProtector_get_num_protected_objects, NULL,
-        DEBUG_GC_PROTECTOR_NUM_PROTECTED_OBJECTS_DOC, NULL },
-      { "num_active_protections", (getter) DebugGCProtector_get_num_active_protections, NULL,
-        DEBUG_GC_PROTECTOR_NUM_ACTIVE_PROTECTIONS_DOC, NULL },
+PyGetSetDef  RaisingGCProtector_properties[]
+  = { { "num_protected_objects", (getter) RaisingGCProtector_get_num_protected_objects, NULL,
+        RAISING_GC_PROTECTOR_NUM_PROTECTED_OBJECTS_DOC, NULL },
+      { "num_active_protections", (getter) RaisingGCProtector_get_num_active_protections, NULL,
+        RAISING_GC_PROTECTOR_NUM_ACTIVE_PROTECTIONS_DOC, NULL },
       { NULL, NULL, NULL, NULL, NULL } };
 
+PyTypeObject  RaisingGCProtector_Type
+  = { PyObject_HEAD_INIT (NULL)
+      0,                                             /* ob_size           */
+      "notify.gc.RaisingGCProtector",                /* tp_name           */
+      sizeof (RaisingGCProtector),                   /* tp_basicsize      */
+      0,                                             /* tp_itemsize       */
+      (destructor)     RaisingGCProtector_dealloc,   /* tp_dealloc        */
+      (printfunc)      0,                            /* tp_print          */
+      (getattrfunc)    0,                            /* tp_getattr        */
+      (setattrfunc)    0,                            /* tp_setattr        */
+      (cmpfunc)        0,                            /* tp_compare        */
+      (reprfunc)       0,                            /* tp_repr           */
+      0,                                             /* tp_as_number      */
+      0,                                             /* tp_as_sequence    */
+      0,                                             /* tp_as_mapping     */
+      (hashfunc)       0,                            /* tp_hash           */
+      (ternaryfunc)    0,                            /* tp_call           */
+      (reprfunc)       0,                            /* tp_str            */
+      (getattrofunc)   0,                            /* tp_getattro       */
+      (setattrofunc)   0,                            /* tp_setattro       */
+      0,                                             /* tp_as_buffer      */
+      Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,      /* tp_flags          */
+      RAISING_GC_PROTECTOR_DOC,                      /* tp_doc            */
+      (traverseproc)   0,                            /* tp_traverse       */
+      (inquiry)        0,                            /* tp_clear          */
+      (richcmpfunc)    0,                            /* tp_richcompare    */
+      0,                                             /* tp_weaklistoffset */
+      (getiterfunc)    0,                            /* tp_iter           */
+      (iternextfunc)   0,                            /* tp_iternext       */
+      RaisingGCProtector_methods,                    /* tp_methods        */
+      0,                                             /* tp_members        */
+      RaisingGCProtector_properties,                 /* tp_getset         */
+      (PyTypeObject *) &AbstractGCProtector_Type,    /* tp_base           */
+      (PyObject *)     0,                            /* tp_dict           */
+      0,                                             /* tp_descr_get      */
+      0,                                             /* tp_descr_set      */
+      0,                                             /* tp_dictoffset     */
+      (initproc)       RaisingGCProtector_init,      /* tp_init           */
+      (allocfunc)      0,                            /* tp_alloc          */
+      (newfunc)        0,                            /* tp_new            */
+      (freefunc)       0,                            /* tp_free           */
+      (inquiry)        0,                            /* tp_is_gc          */
+      (PyObject *)     0,                            /* tp_bases          */
+    };
+
+
+PyMethodDef  DebugGCProtector_methods[]
+  = { { "unprotect",   (PyCFunction) DebugGCProtector_unprotect,
+        METH_VARARGS | METH_KEYWORDS, DEBUG_GC_PROTECTOR_UNPROTECT_DOC },
+      { NULL, NULL, 0, NULL } };
+
 PyTypeObject  DebugGCProtector_Type
   = { PyObject_HEAD_INIT (NULL)
       0,                                             /* ob_size           */
-      "notify.gc.DebugGCProtector",                  /* tp_name           */
+      "notify.gc.DebugGCProtector_Type",             /* tp_name           */
       sizeof (DebugGCProtector),                     /* tp_basicsize      */
       0,                                             /* tp_itemsize       */
-      (destructor)     DebugGCProtector_dealloc,     /* tp_dealloc        */
+      (destructor)     0,                            /* tp_dealloc        */
       (printfunc)      0,                            /* tp_print          */
       (getattrfunc)    0,                            /* tp_getattr        */
       (setattrfunc)    0,                            /* tp_setattr        */
@@ -346,13 +428,13 @@ PyTypeObject  DebugGCProtector_Type
       (iternextfunc)   0,                            /* tp_iternext       */
       DebugGCProtector_methods,                      /* tp_methods        */
       0,                                             /* tp_members        */
-      DebugGCProtector_properties,                   /* tp_getset         */
-      (PyTypeObject *) &AbstractGCProtector_Type,    /* tp_base           */
+      0,                                             /* tp_getset         */
+      (PyTypeObject *) &RaisingGCProtector_Type,     /* tp_base           */
       (PyObject *)     0,                            /* tp_dict           */
       0,                                             /* tp_descr_get      */
       0,                                             /* tp_descr_set      */
       0,                                             /* tp_dictoffset     */
-      (initproc)       DebugGCProtector_init,        /* tp_init           */
+      (initproc)       0,                            /* tp_init           */
       (allocfunc)      0,                            /* tp_alloc          */
       (newfunc)        0,                            /* tp_new            */
       (freefunc)       0,                            /* tp_free           */
@@ -369,6 +451,8 @@ static PyObject *  raise_not_implemented_exception;
 static char *      no_keywords[]     = { NULL };
 static char *      object_keywords[] = { "object", NULL };
 
+static PyObject *  unprotection_exception_type;
+
 
 
 /*- AbstractGCProtector type methods -------------------------------*/
@@ -504,12 +588,12 @@ FastGCProtector_get_num_active_protections (FastGCProtector *self)
 
 
 
-/*- DebugGCProtector type methods ----------------------------------*/
+/*- RaisingGCProtector type methods --------------------------------*/
 
 static int
-DebugGCProtector_init (DebugGCProtector *self, PyObject *arguments, PyObject *keywords)
+RaisingGCProtector_init (RaisingGCProtector *self, PyObject *arguments, PyObject *keywords)
 {
-  if (!PyArg_ParseTupleAndKeywords (arguments, keywords, ":notify.gc.DebugGCProtector",
+  if (!PyArg_ParseTupleAndKeywords (arguments, keywords, ":notify.gc.RaisingGCProtector",
                                     no_keywords))
     return -1;
 
@@ -521,7 +605,7 @@ DebugGCProtector_init (DebugGCProtector *self, PyObject *arguments, PyObject *ke
 
 
 static void
-DebugGCProtector_dealloc (DebugGCProtector *self)
+RaisingGCProtector_dealloc (RaisingGCProtector *self)
 {
   Py_DECREF (self->protected_objects_dict);
   self->ob_type->tp_free ((PyObject *) self);
@@ -529,12 +613,12 @@ DebugGCProtector_dealloc (DebugGCProtector *self)
 
 
 static PyObject *
-DebugGCProtector_protect (DebugGCProtector *self, PyObject *arguments, PyObject *keywords)
+RaisingGCProtector_protect (RaisingGCProtector *self, PyObject *arguments, PyObject *keywords)
 {
   PyObject *object;
 
   if (!PyArg_ParseTupleAndKeywords (arguments, keywords,
-                                    "O:notify.gc.DebugGCProtector.protect",
+                                    "O:notify.gc.RaisingGCProtector.protect",
                                     object_keywords, &object))
     return NULL;
 
@@ -569,12 +653,12 @@ DebugGCProtector_protect (DebugGCProtector *self, PyObject *arguments, PyObject
 
 
 static PyObject *
-DebugGCProtector_unprotect (DebugGCProtector *self, PyObject *arguments, PyObject *keywords)
+RaisingGCProtector_unprotect (RaisingGCProtector *self, PyObject *arguments, PyObject *keywords)
 {
   PyObject *object;
 
   if (!PyArg_ParseTupleAndKeywords (arguments, keywords,
-                                    "O:notify.gc.DebugGCProtector.unprotect",
+                                    "O:notify.gc.RaisingGCProtector.unprotect",
                                     object_keywords, &object))
     return NULL;
 
@@ -603,11 +687,25 @@ DebugGCProtector_unprotect (DebugGCProtector *self, PyObject *arguments, PyObjec
         }
       else
         {
-          PyErr_SetString (PyExc_ValueError, "object is not protected by this DebugGCProtector");
-          PyErr_Print ();
+          const char *type_name = self->ob_type->tp_name;
 
-          /* So that the return statement at the end does no implicit unprotection. */
-          Py_INCREF (object);
+          if (type_name)
+            {
+              type_name = strrchr (type_name, '.');
+
+              if (type_name)
+                type_name += 1;
+              else
+                type_name = self->ob_type->tp_name;
+            }
+          else
+            type_name = "?";
+
+          PyErr_Format (unprotection_exception_type,
+                        "object is not protected by this %s", type_name);
+
+          /* Cause exception to be raised. */
+          object = NULL;
         }
 
       Py_DECREF (id);
@@ -624,15 +722,15 @@ DebugGCProtector_unprotect (DebugGCProtector *self, PyObject *arguments, PyObjec
 
 
 static PyObject *
-DebugGCProtector_get_num_object_protections (DebugGCProtector *self,
-                                             PyObject *arguments, PyObject *keywords)
+RaisingGCProtector_get_num_object_protections (RaisingGCProtector *self,
+                                               PyObject *arguments, PyObject *keywords)
 {
   PyObject *object;
   PyObject *id;
   PyObject *num_protections;
 
   if (!PyArg_ParseTupleAndKeywords (arguments, keywords,
-                                    "O:notify.gc.DebugGCProtector.unprotect",
+                                    "O:notify.gc.RaisingGCProtector.unprotect",
                                     object_keywords, &object))
     return NULL;
 
@@ -650,19 +748,48 @@ DebugGCProtector_get_num_object_protections (DebugGCProtector *self,
 
 
 static PyObject *
-DebugGCProtector_get_num_protected_objects (DebugGCProtector *self)
+RaisingGCProtector_get_num_protected_objects (RaisingGCProtector *self)
 {
   return PyInt_FromLong (PyDict_Size (self->protected_objects_dict));
 }
 
 
 static PyObject *
-DebugGCProtector_get_num_active_protections (DebugGCProtector *self)
+RaisingGCProtector_get_num_active_protections (RaisingGCProtector *self)
 {
   return PyInt_FromLong (self->num_active_protections);
 }
 
 
+
+/*- DebugGCProtector type methods ----------------------------------*/
+
+static PyObject *
+DebugGCProtector_unprotect (DebugGCProtector *self, PyObject *arguments, PyObject *keywords)
+{
+  PyObject *object;
+
+  /* For the proper exception message and so we can assume that if super unprotect()
+   * method fails, than it is because object is not protected.
+   */
+  if (!PyArg_ParseTupleAndKeywords (arguments, keywords,
+                                    "O:notify.gc.DebugGCProtector.unprotect",
+                                    object_keywords, &object))
+    return NULL;
+
+  if (!RaisingGCProtector_unprotect (self, arguments, keywords))
+    {
+      PyErr_Print ();
+      PyErr_Clear ();
+
+      /* So that the return statement does no implicit unprotection. */
+      Py_INCREF (object);
+    }
+
+  return object;
+}
+
+
 
 /*- Module initialization ------------------------------------------*/
 
@@ -693,8 +820,13 @@ initgc (void)
   raise_not_implemented_exception = PyDict_GetItemString (PyModule_GetDict (utilities),
                                                           "raise_not_implemented_exception");
 
+  unprotection_exception_type     = PyErr_NewException ("notify.gc.UnprotectionError",
+                                                        PyExc_ValueError, NULL);
+  PyDict_SetItemString (dictionary, "UnprotectionError", unprotection_exception_type);
+
   REGISTER_TYPE (dictionary, AbstractGCProtector_Type, "AbstractGCProtector");
   REGISTER_TYPE (dictionary, FastGCProtector_Type,     "FastGCProtector");
+  REGISTER_TYPE (dictionary, RaisingGCProtector_Type,  "RaisingGCProtector");
   REGISTER_TYPE (dictionary, DebugGCProtector_Type,    "DebugGCProtector");
 
   default_protector = FastGCProtector_new ();
