@@ -110,7 +110,7 @@ import weakref
 
 from notify.bind  import Binding, WeakBinding
 from notify.gc    import AbstractGCProtector
-from notify.utils import raise_not_implemented_exception
+from notify.utils import is_callable, as_string, raise_not_implemented_exception
 
 try:
     import contextlib
@@ -917,7 +917,7 @@ class Signal (AbstractSignal):
     # references must be a really weird thing to do, so if you really
     # need them, you should use your own `Signal' subclass.
     __slots__ = ('_handlers', '_blocked_handlers',
-                 '_Signal__accumulator', '_Signal__emission_level')
+                 as_string.__accumulator, as_string.__emission_level)
 
 
     def __init__(self, accumulator = None):
@@ -934,7 +934,7 @@ class Signal (AbstractSignal):
         """
 
         if not (accumulator is None or isinstance (accumulator, Signal.AbstractAccumulator)):
-            raise TypeError ("you must provide a `Signal.AbstractAccumulator' or None")
+            raise TypeError ("you must provide a 'Signal.AbstractAccumulator' or None")
 
         super (Signal, self).__init__()
 
@@ -976,7 +976,7 @@ class Signal (AbstractSignal):
 
 
     def is_connected (self, handler, *arguments):
-        if self._handlers is not None and callable (handler):
+        if self._handlers is not None and is_callable (handler):
             if arguments:
                 handler = Binding (handler, arguments)
 
@@ -987,7 +987,7 @@ class Signal (AbstractSignal):
 
 
     def is_blocked (self, handler, *arguments):
-        if self._blocked_handlers is not _EMPTY_TUPLE and callable (handler):
+        if self._blocked_handlers is not _EMPTY_TUPLE and is_callable (handler):
             if arguments:
                 handler = Binding (handler, arguments)
 
@@ -1011,7 +1011,7 @@ class Signal (AbstractSignal):
 
     def disconnect (self, handler, *arguments):
         handlers = self._handlers
-        if handlers is None or not callable (handler):
+        if handlers is None or not is_callable (handler):
             return False
 
         if arguments:
@@ -1052,7 +1052,7 @@ class Signal (AbstractSignal):
     # Overriden for efficiency.
 
     def disconnect_all (self, handler, *arguments):
-        if self._handlers is None or not callable (handler):
+        if self._handlers is None or not is_callable (handler):
             return False
 
         if arguments:
@@ -1088,7 +1088,7 @@ class Signal (AbstractSignal):
 
 
     def block (self, handler, *arguments):
-        if callable (handler) and self._handlers is not None:
+        if is_callable (handler) and self._handlers is not None:
             if arguments:
                 handler = Binding (handler, arguments)
 
@@ -1104,7 +1104,7 @@ class Signal (AbstractSignal):
 
 
     def unblock (self, handler, *arguments):
-        if self._blocked_handlers is _EMPTY_TUPLE or not callable (handler):
+        if self._blocked_handlers is _EMPTY_TUPLE or not is_callable (handler):
             return False
 
         if arguments:
@@ -1135,10 +1135,12 @@ class Signal (AbstractSignal):
             try:
                 saved_emission_level  = self.__emission_level
                 self.__emission_level = abs (saved_emission_level) + 1
+                might_have_garbage    = False
 
                 for handler in handlers:
                     # Disconnected while in emission handlers are temporary set to None.
                     if handler is None or handler in self._blocked_handlers:
+                        might_have_garbage = (handler is None)
                         continue
 
                     # This somewhat illogical transposition of terms is for speed
@@ -1146,28 +1148,34 @@ class Signal (AbstractSignal):
                     # doesn't matter which term is evaluated first.
                     if not handler and isinstance (handler, WeakBinding):
                         # Handler will be removed in collect_garbage(), don't bother now.
+                        might_have_garbage = True
                         continue
 
                     if self.__emission_level < 0:
+                        might_have_garbage = True
                         break
 
-                    try:
-                        handler_value = handler (*arguments)
-
-                    # To also catch old-style string-only exceptions.
-                    except:
-                        AbstractSignal.exception_handler (self, sys.exc_info () [1], handler)
-                        continue
-
-                    if accumulator is not None:
-                        value = accumulator.accumulate_value (value, handler_value)
-
-                        if not accumulator.should_continue (value):
-                            break
-
+                    # Another speed optimization, check if we even need that
+                    # `handler_value' first.
+                    if accumulator is None:
+                        try:
+                            handler (*arguments)
+                        except:
+                            AbstractSignal.exception_handler (self, sys.exc_info () [1], handler)
+                    else:
+                        try:
+                            handler_value = handler (*arguments)
+                        except:
+                            AbstractSignal.exception_handler (self, sys.exc_info () [1], handler)
+                        else:
+                            value = accumulator.accumulate_value (value, handler_value)
+                            if not accumulator.should_continue (value):
+                                might_have_garbage = True
+                                break
             finally:
                 self.__emission_level = saved_emission_level
-                self.collect_garbage ()
+                if might_have_garbage and saved_emission_level == 0:
+                    self.collect_garbage ()
 
         if accumulator is None:
             return None
@@ -1197,12 +1205,10 @@ class Signal (AbstractSignal):
         # Don't remove disconnected or garbage-collected handlers if in nested emission,
         # it will spoil emit() calls completely.
         if self._handlers is not None and self.__emission_level == 0:
-            self._handlers = [handler for handler in self._handlers
-                              if handler is not None and (not isinstance (handler, WeakBinding)
-                                                          or handler)]
-
-            if not self._handlers:
-                self._handlers = None
+            self._handlers = ([handler for handler in self._handlers
+                               if handler is not None and (not isinstance (handler, WeakBinding)
+                                                           or handler)]
+                              or None)
 
 
     def _additional_description (self, formatter):
@@ -1226,7 +1232,7 @@ class CleanSignal (Signal):
     least one handler.
     """
 
-    __slots__ = ('_CleanSignal__parent', '__weakref__')
+    __slots__ = (as_string.__parent, '__weakref__')
 
 
     def __init__(self, parent = None, accumulator = None):
