@@ -41,6 +41,7 @@ from notify.base      import AbstractValueObject
 from notify.condition import AbstractStateTrackingCondition
 from notify.gc        import AbstractGCProtector
 from notify.signal    import CleanSignal
+from notify.utils     import is_callable, as_string
 
 
 
@@ -104,7 +105,7 @@ class AbstractValueTrackingVariable (AbstractVariable):
     mutable variable implementation, see C{L{Variable}} class.
     """
 
-    __slots__ = ('_AbstractValueTrackingVariable__value')
+    __slots__ = (as_string.__value)
 
 
     def __init__(self, initial_value = None):
@@ -120,7 +121,7 @@ class AbstractValueTrackingVariable (AbstractVariable):
         """
 
         if not self.is_allowed_value (initial_value):
-            raise ValueError ("`%s' is not allowed as value of the variable" % initial_value)
+            raise ValueError ("'%s' is not allowed as value of the variable" % initial_value)
 
         super (AbstractValueTrackingVariable, self).__init__()
         self.__value = initial_value
@@ -157,7 +158,7 @@ class AbstractValueTrackingVariable (AbstractVariable):
 
         if self.get () != value:
             if not self.is_allowed_value (value):
-                raise ValueError ("`%s' is not allowed as value of the variable" % value)
+                raise ValueError ("'%s' is not allowed as value of the variable" % value)
 
             self.__value = value
             return self._value_changed (value)
@@ -192,15 +193,21 @@ class AbstractValueTrackingVariable (AbstractVariable):
 
             for allowed_type in allowed_value_types:
                 if not isinstance (allowed_type, (type, types.ClassType)):
-                    raise TypeError ("`allowed_value_types' must be a tuple of types and classes")
+                    raise TypeError ("'allowed_value_types' must be a tuple of types and classes")
 
         for attribute in (super (AbstractValueTrackingVariable, cls)
                           ._generate_derived_type_dictionary (options)):
             if attribute[0] not in ('get', 'set'):
                 yield attribute
 
-        functions = {}
-        object    = options.get ('object')
+        functions        = {}
+        object           = options.get ('object')
+        filtered_options = AbstractValueObject._filter_options (options,
+                                                                'cls',
+                                                                'allowed_values',
+                                                                'allowed_value_types',
+                                                                'getter', 'setter',
+                                                                'default_value')
 
         if allowed_values is not None or allowed_value_types is not None:
             if allowed_values is not None:
@@ -208,13 +215,14 @@ class AbstractValueTrackingVariable (AbstractVariable):
                     exec ('def is_allowed_value (self, value):\n'
                           '    return value in allowed_values\n'
                           '           and isinstance (value, allowed_value_types)') \
-                          in options, functions
+                          in filtered_options, functions
                 else:
                     exec ('def is_allowed_value (self, value):\n'
-                          '    return value in allowed_values\n') in options, functions
+                          '    return value in allowed_values\n') in filtered_options, functions
             else:
                 exec ('def is_allowed_value (self, value):\n'
-                      '    return isinstance (value, allowed_value_types)') in options, functions
+                      '    return isinstance (value, allowed_value_types)') \
+                      in filtered_options, functions
 
         if 'getter' in options:
             if object is not None:
@@ -222,20 +230,20 @@ class AbstractValueTrackingVariable (AbstractVariable):
                        '    cls.__init__(self, getter (%s))\n'
                        '    %s = %s')
                       % (object, object, AbstractValueObject._get_object (options), object)) \
-                      in options, functions
+                      in filtered_options, functions
             else:
                 exec ('def __init__(self):\n'
-                      '    cls.__init__(self, getter (self))\n') in options, functions
+                      '    cls.__init__(self, getter (self))\n') in filtered_options, functions
 
             exec (('def resynchronize_with_backend (self):\n'
                    '    self._set (getter (%s))')
-                  % AbstractValueObject._get_object (options)) in options, functions
+                  % AbstractValueObject._get_object (options)) in filtered_options, functions
 
         else:
             if 'default_value' in options:
                 if (    'is_allowed_value' in functions
                     and not functions['is_allowed_value'] (None, options['default_value'])):
-                   raise ValueError ("`default_value' of `%s' is not within allowed value set"
+                   raise ValueError ("'default_value' of '%s' is not within allowed value set"
                                      % (options['default_value'],))
 
                 initial_default = ' = default_value'
@@ -261,34 +269,31 @@ class AbstractValueTrackingVariable (AbstractVariable):
                        '    %s\n')
                       % (object, initial_default,
                          AbstractValueObject._get_object (options), object), setter_statement) \
-                      in options, functions
+                      in filtered_options, functions
             else:
                 exec (('def __init__(self, initial_value%s):\n'
                        '    cls.__init__(self, initial_value)\n'
                        '    %s\n')
-                      % (initial_default, setter_statement)) in options, functions
+                      % (initial_default, setter_statement)) in filtered_options, functions
 
         if 'setter' in options:
             exec (('def _set (self, value):\n'
                    '    if self.get () != value:\n'
                    '        if not self.is_allowed_value (value):\n'
                    '            raise ValueError \\\n'
-                   '                ("`%%s\' is not allowed as value of the variable" %% value)\n'
+                   '                ("\'%%s\' is not allowed as value of the variable" %% value)\n'
                    '        setter (%s, value)\n'
                    '        self._AbstractValueTrackingVariable__value = value\n'
                    '        return self._value_changed (value)\n'
                    '    else:\n'
                    '        return False')
-                  % AbstractValueObject._get_object (options)) in options, functions
+                  % AbstractValueObject._get_object (options)) in filtered_options, functions
 
-            exec 'def set (self, value): return self._set (value)' in options, functions
+            exec 'def set (self, value): return self._set (value)' in functions
 
         for function in functions.iteritems ():
             yield function
 
-        del functions
-        del object
-
 
     _generate_derived_type_dictionary = classmethod (_generate_derived_type_dictionary)
 
@@ -352,7 +357,7 @@ class WatcherVariable (AbstractValueTrackingVariable):
     @see:  condition.WatcherCondition
     """
 
-    __slots__ = ('_WatcherVariable__watched_variable')
+    __slots__ = (as_string.__watched_variable)
 
 
     def __init__(self, variable_to_watch = None):
@@ -474,7 +479,7 @@ class WatcherVariable (AbstractValueTrackingVariable):
 
 
     def _generate_derived_type_dictionary (cls, options):
-        raise TypeError ("`WatcherVariable' doesn't support derive_type() method")
+        raise TypeError ("'WatcherVariable' doesn't support derive_type() method")
 
     _generate_derived_type_dictionary = classmethod (_generate_derived_type_dictionary)
 
@@ -484,11 +489,11 @@ class WatcherVariable (AbstractValueTrackingVariable):
 
 class _PredicateOverVariable (AbstractStateTrackingCondition):
 
-    __slots__ = ('_PredicateOverVariable__predicate', '_PredicateOverVariable__variable')
+    __slots__ = (as_string.__predicate, as_string.__variable)
 
 
     def __init__(self, predicate, variable):
-        if not callable (predicate):
+        if not is_callable (predicate):
             raise TypeError ('predicate must be callable')
 
         super (_PredicateOverVariable, self).__init__(predicate (variable.get ()))
