@@ -33,14 +33,15 @@ import unittest
 
 import types
 
-from notify.all import *
+from notify.all    import *
+from test.__common import NotifyTestCase
 
 
 
-class AllTestCase (unittest.TestCase):
+class AllTestCase (NotifyTestCase):
 
     def assert_is_function (self, function):
-        self.assert_(isinstance (function, types.FunctionType))
+        self.assert_(isinstance (function, (types.FunctionType, types.BuiltinFunctionType)))
 
 
     def assert_is_class (self, _class):
@@ -60,10 +61,7 @@ class AllTestCase (unittest.TestCase):
                 except:
                     continue
 
-                def set_non_existing_attribute ():
-                    object.this_attribute_sure_doesnt_exist = None
-
-                self.assertRaises (AttributeError, set_non_existing_attribute)
+                self.assertRaises (AttributeError, self.non_existing_attribute_setter (object))
                 break
 
 
@@ -110,8 +108,14 @@ class AllTestCase (unittest.TestCase):
 
 
     def test_util (self):
-        self.assert_is_function (raise_not_implemented_exception)
+        self.assert_is_function (is_callable)
         self.assert_is_function (is_valid_identifier)
+        self.assert_is_function (mangle_identifier)
+
+         # It is not a function, not a class...  Just test it is there.
+        self.assert_            (as_string)
+
+        self.assert_is_function (raise_not_implemented_exception)
         self.assert_is_class    (DummyReference)
 
     def test_variable (self):
